# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.drivers.au.base import AuQuery, AuOrder, AuRerunOrder, AuBaseDriver
from driverng.fixed import Date, FixedLengthFormat, Multiple, String

logger = logging.getLogger("driverng.au5800")


class Au5800Result(FixedLengthFormat):
    TEST = [
        # 0 or 1
        String("cuvette", 1),
        String("test_number", 3),
        String("reagent", 16),
        String("analysis_data", 9),
        String("data_flags", 8),
    ]
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + AuOrder.PATIENT
        # YYYYMMDDHHMMSS
        + [Date("loading_time", 14)]
        + [Multiple("tests", TEST)]
    )


class Au5800RerunResult(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Date("loading_time", 14)]
        + [Multiple("tests", Au5800Result.TEST)]
    )


class Au5800QC(FixedLengthFormat):
    TEST = [
        String("test_number", 3),
        String("reagent", 16),
        String("analysis_data", 9),
        String("data_flags", 8),
    ]
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Date("loading_time", 14), String("unit", 1), String("cuvette", 1)]
        + [Multiple("tests", TEST)]
    )


class Au5800Driver(AuBaseDriver):
    SPEC = {"code": "au5800", "label": "AU5800"}

    @staticmethod
    def get_klass(code):
        if code in ("Sh", "SH"):
            return AuRerunOrder
        elif code[0] == "S":
            return AuOrder
        elif code[0] == "R":
            return AuQuery
        elif code in ("DQ", "DR", "DA"):
            return Au5800QC
        elif code in ("Dh", "DH"):
            return Au5800RerunResult
        elif code[0] == "D":
            return Au5800Result
        else:
            raise NotImplementedError

    @staticmethod
    def get_result_worlist(message, worklist):
        rack = ""
        if message.rack and message.cup:
            rack = "{}_{}".format(message.rack, message.cup)
        elif message.rack and not message.cup:
            rack = message.rack
        return worklist.get_or_create(
            type="patient",
            sample_id=message.sample_id,
            specimen=message.sample_type or "S",
            rack=rack,
        )

    def decode_message(self, message):
        if isinstance(message, AuQuery):
            return self.get_query_worklist(message)

        worklist = Worklist(context="result")
        if isinstance(message, Au5800Result) or isinstance(message, Au5800RerunResult):
            sample = self.get_result_worlist(message, worklist)
        elif isinstance(message, Au5800QC):
            sample = self.get_qc_worklist(message, worklist)
        else:
            raise NotImplementedError

        for test in message.tests:
            flags = []
            if test.data_flags:
                for flag in test.data_flags:
                    if flag.strip():
                        flags.append(flag)
            lots = []
            reagents = self.get_reagents(test.reagent)
            for reagent in reagents:
                lots.append({"lot": reagent, "type": "reagent"})
            sample.results.add(
                code=test.test_number,
                value=test.analysis_data,
                date=message.loading_time,
                flags=flags,
                lots=lots,
            )
            if test.test_number == "096":
                splitted = test.analysis_data.split()
                if len(splitted) == 3:
                    for code, value in zip(["L", "I", "H"], splitted):
                        sample.results.add(
                            code=code,
                            value=value,
                            date=message.loading_time,
                            flags=flags,
                            lots=lots,
                        )

        return worklist
